const EVENT_DELAY = 200;
const HEIGHT_INCREMENT = 20;
const SORT_ARRAY = [16,11,4,5,3,7,10,8,9,2,1];
 

const store = new Vuex.Store({
  state: {
    values: [],
    cards: [],
    done: true,

	strValues:[],
  },
  
  mutations: {

    reset (state, payload) {
      state.values = payload.values;

	  state.values.forEach((item,i)=>state.strValues.push(item+'&'+i));

      state.cards = [];
      for (let i = 0; i < state.values.length; i++) {
        state.cards.push({
          value:state.values[i],
		  strValue:state.strValues[i],
          sortIndex:i,
          isActive: false,
          isLocked: false
        });
      }
      
      state.done = false;
    },

    swap (state, payload) {
      let a = payload.indexes[0]; 
      let b = payload.indexes[1]; 
      let temp = state.values[a];

      state.values[a] = state.values[b];
      state.values[b] = temp;

	  let temp2 = state.strValues[a];
      state.strValues[a] = state.strValues[b];
      state.strValues[b] = temp2;

      state.cards.forEach((card) => {
			card.sortIndex =state.strValues.indexOf(card.strValue);
      });
    },

    activate (state, payload) {
      payload.indexes.forEach((index) => {
			state.cards.forEach((card) => {
				if(card.sortIndex === index) card.isActive = true;
			});
      });
    },

    deactivate (state, payload) {
      payload.indexes.forEach((index) => {
			state.cards.forEach((card) => {
				if(card.sortIndex === index) card.isActive = false;
			});
      });
    },

    lock (state, payload) {
      payload.indexes.forEach((index) => {
			state.cards.forEach((card) => {
				if(card.sortIndex === index) card.isLocked = true;
			});
      });
    },

    done (state) {
      state.done = true;
    }
  }
});

Vue.component('sort-card', {
  template: '#sort-card-template',
  props: ['value', 'sortIndex', 'isActive', 'isLocked'],
  computed: {
    cardClassObject() {
      return {
        'card-active': this.isActive,
        'card-locked': this.isLocked
      }
    }
  }
});

new Vue({
  el: '#app',
  store, 
  created(){
    this.reset(SORT_ARRAY);
  },
  methods: {

    reset(arr) {

	  let values = arr.slice();
      store.commit({ type: 'reset', values: values });

      let sequence = this.bubbleSort(values);

      sequence.forEach((event, index) => {
        setTimeout(() => { store.commit(event); }, index * EVENT_DELAY);
      });
    },

    bubbleSort(values) {

      let sequence = [];

      let swapped;

      let indexLastUnsorted = values.length - 1;

      do {
        swapped = false;
        for (let i = 0; i < indexLastUnsorted; i++) {

          sequence.push({ type: 'activate', indexes: [i, i + 1] });

          if (values[i] > values[i + 1]) {
            let temp = values[i];
            values[i] = values[i + 1];
            values[i + 1] = temp;
            swapped = true;

            sequence.push({ type: 'swap', indexes: [i, i + 1] });
          }

          sequence.push({ type: 'deactivate', indexes: [i, i + 1] });
        }

        sequence.push({ type: 'lock', indexes: [indexLastUnsorted] });
        indexLastUnsorted--;
      } while (swapped);
      

      let skipped = Array.from(Array(indexLastUnsorted + 1).keys());
      sequence.push({ type: 'lock', indexes: skipped });
      sequence.push({ type: 'done' });
	  console.log('An array containing the content of each step',sequence);
      return sequence;
    }
  }
});